#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.

# interface_implementation.py - Example of implementing an interface

#--[Imports]-------------------------------------------------------------------
from traits.api import HasTraits, implements, Str, Instance
from interface_definition import IName


#--[Code]----------------------------------------------------------------------
class Person(HasTraits):
    implements(IName)

    first_name = Str('John')
    last_name = Str('Doe')

    # Implementation of the 'IName' interface:
    def get_name(self):
        """ Returns the name of an object. """
        return ('%s %s' % (self.first_name, self.last_name))


#--[Example*]------------------------------------------------------------------
class Apartment(HasTraits):
    renter = Instance(IName)

william = Person(first_name='William', last_name='Adams')
apt1 = Apartment(renter=william)
print 'Renter is: ', apt1.renter.get_name()
# Result: Renter is: William Adams



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.

# metadata.py --- Example of accessing trait metadata attributes

#--[Imports]-------------------------------------------------------------------
from traits.api import (HasTraits, Int, List, Float, Instance, Any, TraitType)


#--[Code]----------------------------------------------------------------------
class Foo(HasTraits):
    pass


class Test(HasTraits):
    i = Int(99)
    lf = List(Float)
    foo = Instance(Foo, ())
    any = Any([1, 2, 3])


#--[Example*]------------------------------------------------------------------
t = Test()

# Prints values of various metadata attributes for each of the traits.
print t.trait('i').default                      # 99
print t.trait('i').default_kind                 # value
print t.trait('i').inner_traits                 # ()
print t.trait('i').is_trait_type(Int)           # True
print t.trait('i').is_trait_type(Float)         # False

print t.trait('lf').default                     # []
print t.trait('lf').default_kind                # list
print t.trait('lf').inner_traits  # (<traits.traits.CTrait object at
                                  #  0x01B24138>,)
print t.trait('lf').is_trait_type(List)         # True
print t.trait('lf').is_trait_type(TraitType)    # True
print t.trait('lf').is_trait_type(Float)        # False
print t.trait('lf').inner_traits[0].is_trait_type(Float)  # True

print t.trait('foo').default                    # <undefined>
print t.trait('foo').default_kind               # factory
print t.trait('foo').inner_traits               # ()
print t.trait('foo').is_trait_type(Instance)    # True
print t.trait('foo').is_trait_type(List)        # False

print t.trait('any').default                    # [1, 2, 3]
print t.trait('any').default_kind               # list
print t.trait('any').inner_traits               # ()
print t.trait('any').is_trait_type(Any)         # True
print t.trait('any').is_trait_type(List)        # False



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.

# prototype_prefix.py --- Examples of PrototypedFrom() prefix parameter

#--[Imports]-------------------------------------------------------------------
from traits.api import PrototypedFrom, Float, HasTraits, Instance, Str


#--[Code]----------------------------------------------------------------------
class Parent(HasTraits):
    first_name = Str
    family_name = ''
    favorite_first_name = Str
    child_allowance = Float(1.00)


class Child(HasTraits):
    __prefix__ = 'child_'
    first_name = PrototypedFrom('mother', 'favorite_*')
    last_name = PrototypedFrom('father', 'family_name')
    allowance = PrototypedFrom('father', '*')
    father = Instance(Parent)
    mother = Instance(Parent)


#--[Example*]------------------------------------------------------------------
fred = Parent(first_name='Fred',
              family_name='Lopez',
              favorite_first_name='Diego',
              child_allowance=5.0)

maria = Parent(first_name='Maria',
               family_name='Gonzalez',
               favorite_first_name='Tomas',
               child_allowance=10.0)

nino = Child(father=fred, mother=maria)

print '%s %s gets $%.2f for allowance' % (nino.first_name, nino.last_name,
                                          nino.allowance)



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.

# object_trait_attrs.py --- Example of per-object trait attributes

#--[Imports]-------------------------------------------------------------------
from traits.api import HasTraits, Range


#--[Code]----------------------------------------------------------------------
class GUISlider(HasTraits):

    def __init__(self, eval=None, label='Value',
                 trait=None, min=0.0, max=1.0,
                 initial=None, **traits):
        HasTraits.__init__(self, **traits)
        if trait is None:
            if min > max:
                min, max = max, min
            if initial is None:
                initial = min
            elif not (min <= initial <= max):
                initial = [min, max][abs(initial - min) > abs(initial - max)]
            trait = Range(min, max, value=initial)
        self.add_trait(label, trait)



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.

# list_notifier.py -- Example of zero-parameter handlers for an object
#                     containing a list

#--[Imports]-------------------------------------------------------------------
from traits.api import HasTraits, List


#--[Code]----------------------------------------------------------------------
class Employee:
    pass


class Department(HasTraits):
    employees = List(Employee)


#--[Example*]------------------------------------------------------------------
def a_handler():
    print "A handler"


def b_handler():
    print "B handler"


def c_handler():
    print "C handler"

fred = Employee()
mary = Employee()
donna = Employee()

dept = Department(employees=[fred, mary])

# "Old style" name syntax
# a_handler is called only if the list is replaced:
dept.on_trait_change(a_handler, 'employees')
# b_handler is called if the membership of the list changes:
dept.on_trait_change(b_handler, 'employees_items')

# "New style" name syntax
# c_handler is called if 'employees' or its membership change:
dept.on_trait_change(c_handler, '[employees]')

print "Changing list items"
dept.employees[1] = donna     # Calls B and C
print "Replacing list"
dept.employees = [donna]      # Calls A and C



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.

# compound.py -- Example of multiple criteria in a trait definition


#--[Imports]-------------------------------------------------------------------
from traits.api import HasTraits, Range, Trait, TraitError


#--[Code]----------------------------------------------------------------------
# Shows the definition of a compound trait.

class Die(HasTraits):

    # Define a compound trait definition:
    value = Trait(1, Range(1, 6),
                  'one', 'two', 'three', 'four', 'five', 'six')


#--[Example*]------------------------------------------------------------------
# Create a sample Die:
die = Die()

# Try out some sample valid values:
die.value = 3
die.value = 'three'
die.value = 5
die.value = 'five'

# Now try out some invalid values:
try:
    die.value = 0
except TraitError, excp:
    print excp

try:
    die.value = 'zero'
except TraitError, excp:
    print excp



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.

# configure_traits.py -- Sample code to demonstrate configure_traits()

#--[Imports]-------------------------------------------------------------------
from traits.api import HasTraits, Str, Int


#--[Code]----------------------------------------------------------------------
class SimpleEmployee(HasTraits):
    first_name = Str
    last_name = Str
    department = Str

    employee_number = Str
    salary = Int

sam = SimpleEmployee()
sam.configure_traits()



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.

# disallow.py --- Example of using Disallow with wildcards

#--[Imports]-------------------------------------------------------------------
from traits.api import Disallow, Float, HasTraits, Int, Str


#--[Code]----------------------------------------------------------------------
class Person(HasTraits):
    name = Str
    age = Int
    weight = Float
    _ = Disallow



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.


#--[Imports]-------------------------------------------------------------------
from traits.api import HasTraits, Instance


#--[Code]----------------------------------------------------------------------
# Shows the incorrect way of defining mutually-referring classes.
try:
    class Chicken(HasTraits):

        # Won't work: 'Egg' not defined yet:
        hatched_from = Instance(Egg)

    class Egg(HasTraits):

        # If we move this class to the top, then this line won't work, because
        # 'Chicken' won't be defined yet:
        created_by = Instance(Chicken)

except NameError, excp:
    print excp



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.

# simple_adapter.py - Example of adaptation using Adapter

#--[Imports]-------------------------------------------------------------------
from traits.api import Adapter, Instance, implements
from interface_definition import IName
from interface_implementation import Person


#--[Code]----------------------------------------------------------------------
class PersonINameAdapter(Adapter):

    # Declare what interfaces this adapter implements for its
    # client:
    implements(IName)

    # Declare the type of client it supports:
    adaptee = Instance(Person)

    # Implement the 'IName' interface on behalf of its client:
    def get_name(self):
        return ('%s %s' % (self.adaptee.first_name,
                           self.adaptee.last_name))



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.

# bad_self_ref.py -- Non-working example with self-referencing class definition

#--[Imports]-------------------------------------------------------------------
from traits.api import HasTraits, Instance

#--[Code]----------------------------------------------------------------------
# Shows the incorrect way of defining a self-referencing class.
try:
    class Employee(HasTraits):

        # This won't work.
        # 'Employee' is not defined until the class definition is complete:
        manager = Instance(Employee)
except NameError, excp:
    print excp



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.

# use_custom_th.py --- Example of using a custom TraitHandler

#--[Imports]-------------------------------------------------------------------
from traits.api import HasTraits, Trait, TraitRange
from custom_traithandler import TraitOddInteger


#--[Code]----------------------------------------------------------------------
class AnOddClass(HasTraits):
    oddball = Trait(1, TraitOddInteger())
    very_odd = Trait(-1, TraitOddInteger(),
                     TraitRange(-10, -1))



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.

# trait_subclass.py -- Example of subclassing a trait class

from traits.api import BaseInt


#--[Code]----------------------------------------------------------------------
class OddInt(BaseInt):

    # Define the default value
    default_value = 1

    # Describe the trait type
    info_text = 'an odd integer'

    def validate(self, object, name, value):
        value = super(OddInt, self).validate(object, name, value)
        if (value % 2) == 1:
            return value

        self.error(object, name, value)



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.

#--(Extended Property depends_on References)-----------------------------------
"""
Extended Property *depends_on* References
=========================================

In Traits 3.0, the **Property** *depends_on* metadata has been extended to
take advantage of the new extended trait name support offered by the
*on_trait_change* method.

Previously, the *depends_on* metadata for a *Property* was restricted to
referencing traits defined either on the same object as the **Property**, or
on an object immediately reachable from the object. For example::

    class Wheel(Part):

        axel     = Instance(Axel)
        position = Property(depends_on = 'axel.position')

        ...

Starting with Traits 3.0, the *depends_on* metadata may now include any
extended trait reference that is allowed by the enhanced *on_trait_change*
method. So, for example it is now legal to write things like::

    class Wheel(Part):

        axel     = Instance(Axel)
        position = Property(depends_on = 'axel.chassis.position')

or::

    class Child(Person):

        mother = Instance(Person)
        father = Instance(Person)
        mood   = Property(depends_on = ['mother.+mood_affecting',
                                        'father.+mood_affecting'])

In particular, in the last example we are declaring that the **Child** class's
*mood* property depends upon the values of any of either its mother or
father object's traits that have *mood_affecting* metadata defined.

Thus, a **Child** object's *mood* property will fire a trait change
notification whenever any of the its mother's or father's mood affecting
traits change.

Refer also to the code tabs for this lesson for a complete example using a
**Property** definition using *depends_on* metadata containing an extended
trait reference. In particular, take a look at the **LeagueModelView Class**
tab's *total_hits* trait definition.
"""

# FIXME redo example without traitsui

from traits.api import *

from traitsui.api import *

from traitsui.table_column import *


#--[Player Class]--------------------------------------------------------------
# Define a baseball player:
class Player(HasTraits):

    # The name of the player:
    name = Str('<new player>')

    # The number of hits the player made this season:
    hits = Int


#--[Team Class]----------------------------------------------------------------
# Define a baseball team:
class Team(HasTraits):

    # The name of the team:
    name = Str('<new team>')

    # The players on the team:
    players = List(Player)

    # The number of players on the team:
    num_players = Property(depends_on='players')

    def _get_num_players(self):
        """ Implementation of the 'num_players' property.
        """
        return len(self.players)


#--[League Class]--------------------------------------------------------------
# Define a baseball league model:
class League(HasTraits):

    # The name of the league:
    name = Str('<new league>')

    # The teams in the league:
    teams = List(Team)


#--[LeagueModelView Class]-----------------------------------------------------
# Define a ModelView for a League model:
class LeagueModelView(ModelView):

    # The currently selected team:
    team = Instance(Team)

    # The currently selected player:
    player = Instance(Player)

    # Button to add a hit to the current player:
    got_hit = Button('Got a Hit')

    # The total number of hits (note the 'depends_on' extended trait
    # reference):
    total_hits = Property(depends_on='model.teams.players.hits')

    @cached_property
    def _get_total_hits(self):
        """ Returns the total number of hits across all teams and players.
        """
        return reduce(add, [reduce(add, [p.hits for p in t.players], 0)
                            for t in self.model.teams], 0)

    view = View(
        VGroup(
            HGroup(
                Item('total_hits', style='readonly'),
                label='League Statistics',
                show_border=True,
                ),
            VGroup(
                Item('model.teams',
                     show_label=False,
                     editor=TableEditor(
                         columns=[ObjectColumn(name='name',
                                               width=0.70),
                                  ObjectColumn(name='num_players',
                                               label='# Players',
                                               editable=False,
                                               width=0.29)],
                         selected='object.team',
                         auto_add=True,
                         row_factory=Team,
                         configurable=False,
                         sortable=False)
                     ),
                label='League Teams',
                show_border=True,
                ),
            VGroup(
                Item('object.team.players',
                     show_label=False,
                     editor=TableEditor(
                         columns=[ObjectColumn(name='name',
                                               width=0.70),
                                  ObjectColumn(name='hits',
                                               editable=False,
                                               width=0.29)],
                         selected='object.player',
                         auto_add=True,
                         row_factory=Player,
                         configurable=False,
                         sortable=False),
                     ),
                '_',
                HGroup(
                    Item('got_hit',
                         show_label=False,
                         enabled_when='player is not None',
                         ),
                    ),
                label='Team Players',
                show_labels=False,
                show_border=True,
                ),
            ),
        resizable=True,
        )

    def _model_changed(self, model):
        """ Handles the 'league' model being initialized.
        """
        if len(model.teams) > 0:
            self.team = model.teams[0]

    def _got_hit_changed(self):
        """ Handles the currently selected player making a hit.
        """
        self.player.hits += 1

    def _team_changed(self, team):
        """ Handles a new team being selected.
        """
        if len(team.players) > 0:
            self.player = team.players[0]
        else:
            self.player = None

# Function to add two numbers (used with 'reduce'):
add = lambda a, b: a + b

#--[Example*]------------------------------------------------------------------

# Define some sample teams and players:
blue_birds = Team(name='Blue Birds', players=[
    Player(name='Mike Scott', hits=25),
    Player(name='Willy Shofield', hits=37),
    Player(name='Tony Barucci', hits=19)])

chicken_hawks = Team(name='Chicken Hawks', players=[
    Player(name='Jimmy Domore', hits=34),
    Player(name='Bill Janks', hits=16),
    Player(name='Tim Saunders', hits=27)])

eagles = Team(name='Eagles', players=[
    Player(name='Joe Peppers', hits=33),
    Player(name='Sam Alone', hits=12),
    Player(name='Roger Clemson', hits=23)])

# Create a league and its corresponding model view:
demo = LeagueModelView(
    League(name='National Baseball Conference',
           teams=[blue_birds, chicken_hawks, eagles]))



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.

#--(Rewritten Core Traits)-----------------------------------------------------
"""
Rewritten Core Traits
=====================

For several reasons, including the ability to subclass types, many of the
previous core Traits package types have been entirely rewritten as subclasses
of **TraitType**, the new base class for subclassable trait types.

The core trait types which have been rewritten as subclasses of **TraitType**
are:

- Any
- Bool
- CBool
- CComplex
- CInt
- CFloat
- CLong
- Code
- Complex
- CStr
- CUnicode
- Dict
- Directory
- Enum
- Expression
- File
- Float
- HTML
- Instance
- Int
- List
- Long
- Password
- PythonValue
- Range
- Regex
- Str
- String
- Tuple
- Unicode
- WeakRef

This may be useful information if you find yourself in need of creating a new
trait type with behavior similar to any of these core trait types.
"""



#  Copyright (c) 2007, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in /LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt

# cached_prop.py - Example of @cached_property decorator
#--[Imports]-------------------------------------------------------------------
from traits.api import HasPrivateTraits, List, Int, Property, cached_property


#--[Code]----------------------------------------------------------------------
class TestScores(HasPrivateTraits):

    scores = List(Int)
    average = Property(depends_on='scores')

    @cached_property
    def _get_average(self):
        s = self.scores
        return float(reduce(lambda n1, n2: n1 + n2, s, 0)) / len(s)



#------------------------------------------------------------------------------
#  file: refactor_doc.py
#  License: LICENSE.TXT
#
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .function_doc import FunctionDoc
from .class_doc import ClassDoc

#------------------------------------------------------------------------------
# Extension definition
#------------------------------------------------------------------------------
def refactor_docstring(app, what, name, obj, options, lines):

    refactor = None
    if ('class' in what):
        refactor = ClassDoc(lines)
    elif ('function' in what) or ('method' in what):
        refactor = FunctionDoc(lines)

    if refactor is not None:
        refactor.parse()

def setup(app):
    app.setup_extension('sphinx.ext.autodoc')
    app.connect('autodoc-process-docstring', refactor_docstring)



from lib2to3 import fixer_base
from lib2to3.fixer_util import Name


class FixUnicodeMethods(fixer_base.BaseFix):
    """ Custom fixer for the __unicode__ method

    Renames __unicode__ methods to __str__.

    Code used from:
    http://lucumr.pocoo.org/2010/2/11/porting-to-python-3-a-guide/

    """
    PATTERN = r"funcdef< 'def' name='__unicode__' parameters< '(' NAME ')' > any+ >"  # noqa

    def transform(self, node, results):
        name = results['name']
        name.replace(Name('__str__', prefix=name.prefix))



# -*- coding: utf-8 -*-
#
# Traits documentation build configuration file, created by
# sphinx-quickstart on Tue Jul 22 10:52:03 2008.
#
# This file is execfile()d with the current directory set to its containing dir.
#
# The contents of this file are pickled, so don't put values in the namespace
# that aren't pickleable (module imports are okay, they're removed automatically).
#
# All configuration values have a default value; values that are commented out
# serve to show the default value.

import sys
import os

# If your extensions are in another directory, add it here. If the directory
# is relative to the documentation root, use os.path.abspath to make it
# absolute, like shown here.
sys.path.append(os.path.abspath('_extensions'))
sys.path.append(os.path.abspath('../../'))


def mock_modules():
    """ Optionally Mock missing modules to allow autodoc based documentation.

    The ``traits.has_dynamics_view`` imports the traitsui module and
    thus traitsui is needed so that the ``autodoc`` extension can
    extract the docstrings from the has_dynamics_view module. This
    function optionally mocks the traitsui module so that the traits
    documentation can be built without the traitui optional dependency
    installed.

    .. note::

       The mock library is needed in order to mock the traitsui
       package.

    """

    MOCK_MODULES = []
    MOCK_TYPES = []

    # Check to see if we need to mock the traitsui package
    try:
        import traitsui
    except ImportError:
        # Modules that we need to mock
        MOCK_MODULES = [
            'traitsui', 'traitsui.api', 'traitsui.delegating_handler']

        # Collect the types from traitsui that are based on HasTraits
        # We will need to mock them in a special way so that
        # TraitDocumenter can properly identify and document traits.
        from traits.api import HasTraits, HasPrivateTraits
        MOCK_TYPES.append(
            ('traitsui.delegating_handler',
             'DelegatingHandler', (HasTraits,)))
        MOCK_TYPES.append(
            ('traitsui.view_element',
             'ViewSubElement', (HasPrivateTraits,)))
    else:
        return

    try:
        from mock import MagicMock
    except ImportError:
        if len(MOCK_MODULES) != 0:
            print(
                'NOTE: TraitsUI is not installed and mock is not available to '
                'mock the missing modules, some classes will not be documented')
            return

    # Create the custom types for the HasTraits based traitsui objects.
    TYPES = {
        mock_type: type(mock_type, bases, {'__module__': path})
        for path, mock_type, bases in MOCK_TYPES}

    class DocMock(MagicMock):
        """ The special sphinx friendly mocking class to mock missing packages.

        Based on the suggestion from http://docs.readthedocs.org/en/latest/faq.html#i-get-import-errors-on-libraries-that-depend-on-c-modules

        """

        @classmethod
        def __getattr__(self, name):
            if name in ('__file__', '__path__'):
                # sphinx does not like getting a Mock object in this case.
                return '/dev/null'
            else:
                # Return a mock or a custom type as requested.
                return TYPES.get(name, DocMock(mocked_name=name))

        # MagicMock does not define __call__ we do just to make sure
        # that we cover all cases.
        def __call__(self, *args, **kwards):
            return DocMock()

        @property
        def __name__(self):
            # Make sure that if sphinx asks for the name of a Mocked class
            # it gets a nice strings to use (instead of "DocMock")
            return self.mocked_name

    # Add the mocked modules to sys
    sys.modules.update(
        (mod_name, DocMock(mocked_name=mod_name)) for mod_name in MOCK_MODULES)

    # Report on what was mocked.
    print 'mocking modules {0} and types {1}'.format(
        MOCK_MODULES, [mocked[1] for mocked in MOCK_TYPES])

mock_modules()

# General configuration
# ---------------------

# Add any Sphinx extension module names here, as strings. They can be extensions
# coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = [ 'refactordoc',
               'sphinx.ext.viewcode',
               'sphinx.ext.autosummary',
               'traits.util.trait_documenter']

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix of source filenames.
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General substitutions.
project = 'traits'
copyright = '2008-2016, Enthought'

# The default replacements for |version| and |release|, also used in various
# other places throughout the built documents.
d = {}
execfile(os.path.join('..', '..', 'traits', '__init__.py'), d)
version = release = d['__version__']

# There are two options for replacing |today|: either, you set today to some
# non-false value, then it is used:
#today = ''
# Else, today_fmt is used as the format for a strftime call.
today_fmt = '%B %d, %Y'

# List of documents that shouldn't be included in the build.
#unused_docs = []

# List of directories, relative to source directories, that shouldn't be searched
# for source files.
#exclude_dirs = []

# The reST default role (used for this markup: `text`) to use for all documents.
#default_role = None

# If true, '()' will be appended to :func: etc. cross-reference text.
#add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
#add_module_names = True

# If true, sectionauthor and moduleauthor directives will be shown in the
# output. They are ignored by default.
#show_authors = False

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# Options for the autodoc extension.
autodoc_default_flags =['members']

autodoc_member_order = 'bysource'


# Options for HTML output
# -----------------------

# The style sheet to use for HTML and HTML Help pages. A file of that name
# must exist either in Sphinx' static/ path, or in one of the custom paths
# given in html_static_path.
html_style = 'default.css'

# The name for this set of Sphinx documents.  If None, it defaults to
# "<project> v<release> documentation".
html_title = "Traits 4 User Manual"

# A shorter title for the navigation bar.  Default is the same as html_title.
#html_short_title = None

# The name of an image file (within the static path) to place at the top of
# the sidebar.
html_logo = "e-logo-rev.png"

# The name of an image file (within the static path) to use as favicon of the
# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.
html_favicon = "et.ico"

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']

# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
# using the given strftime format.
html_last_updated_fmt = '%b %d, %Y'

# If true, SmartyPants will be used to convert quotes and dashes to
# typographically correct entities.
html_use_smartypants = True

# Custom sidebar templates, maps document names to template names.
#html_sidebars = {}

# Additional templates that should be rendered to pages, maps page names to
# template names.
#html_additional_pages = {}

# If false, no module index is generated.
html_use_modindex = False

# If false, no index is generated.
#html_use_index = False

# If true, the index is split into individual pages for each letter.
#html_split_index = False

# If true, the reST sources are included in the HTML build as _sources/<name>.
#html_copy_source = True

# If true, an OpenSearch description file will be output, and all pages will
# contain a <link> tag referring to it.  The value of this option must be the
# base URL from which the finished HTML is served.
#html_use_opensearch = ''

# If nonempty, this is the file name suffix for HTML files (e.g. ".xhtml").
#html_file_suffix = ''

# Output file base name for HTML help builder.
htmlhelp_basename = 'Traitsdoc'


# Options for LaTeX output
# ------------------------

# The paper size ('letter' or 'a4').
#latex_paper_size = 'letter'

# The font size ('10pt', '11pt' or '12pt').
#latex_font_size = '10pt'

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, document class [howto/manual]).
latex_documents = [
  ('index', 'Traits.tex', 'Traits 4 User Manual', 'Enthought, Inc.', 'manual'),
]

# The name of an image file (relative to this directory) to place at the top of
# the title page.
# latex_logo = "enthought_logo.jpg"
latex_logo = "e-logo-rev.png"

# For "manual" documents, if this is true, then toplevel headings are parts,
# not chapters.
#latex_use_parts = False

# Additional stuff for the LaTeX preamble.
#latex_preamble = ''

# Documents to append as an appendix to all manuals.
#latex_appendices = []

# If false, no module index is generated.
#latex_use_modindex = True

# Options for Texinfo output
# --------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
  ('index', 'traits', 'Traits 4 User Manual', 'Enthought, Inc.',
   'Traits', 'Explicitly typed attributes for Python.', 'Python'),
]



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.

# event.py --- Example of a trait event

#--<Imports>-------------------------------------------------------------------
from traits.api import Event, HasTraits, List, RGBColor, Tuple

#--[Code]----------------------------------------------------------------------

point_2d = Tuple(0, 0)


class Line2D(HasTraits):
    points = List(point_2d)
    line_color = RGBColor('black')
    updated = Event

    def redraw(self):
        pass  # Not implemented for this example

    def _points_changed(self):
        self.updated = True

    def _updated_fired(self):
        self.redraw()



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.

# custom_traithandler.py --- Example of a custom TraitHandler

#--[Imports]-------------------------------------------------------------------
from traits.api import TraitHandler


#--[Code]----------------------------------------------------------------------
class TraitOddInteger(TraitHandler):

    def validate(self, object, name, value):
        if (isinstance(value, int) and
           (value > 0) and ((value % 2) == 1)):
            return value
        self.error(object, name, value)

    def info(self):
        return 'a positive odd integer'



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.

# wildcard.py --- Example of using a wildcard with a trait
#                 attribute name
from traits.api import Any, HasTraits


class Person(HasTraits):
    temp_ = Any



# -*- coding: utf-8 -*-
#------------------------------------------------------------------------------
#  file: base_doc.py
#  License: LICENSE.TXT
#
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
import re

from definition_items import DefinitionItem
from line_functions import is_empty, get_indent, fix_backspace, NEW_LINE


underline_regex = re.compile(r'\s*\S+\s*\Z')

#------------------------------------------------------------------------------
#  Classes
#------------------------------------------------------------------------------

class BaseDoc(object):
    """Base abstract docstring refactoring class.

    The class' main purpose is to parse the dosctring and find the
    sections that need to be refactored. Subclasses should provide
    the methods responsible for refactoring the sections.

    Attributes
    ----------
    docstring : list
        A list of strings (lines) that holds docstrings

    index : int
        The current zero-based line number of the docstring that is currently
        proccessed.

    headers : dict
        The sections that the class refactors. Each entry in the
        dictionary should have as key the name of the section in the
        form that it appears in the docstrings. The value should be
        the postfix of the method, in the subclasses, that is
        responsible for refactoring (e.g. {'Methods': 'method'}).

    BaseDoc also provides a number of methods that operate on the docstring to
    help with the refactoring. This is neccessary because the docstring has to
    change inplace and thus it is better to live the docstring manipulation to
    the class methods instead of accessing the lines directly.

    """

    def __init__(self, lines, headers=None):
        """ Initialize the class

        The method setups the class attributes and starts parsing the
        docstring to find and refactor the sections.

        Arguments
        ---------
        lines : list of strings
            The docstring to refactor

        headers : dict
            The sections for which the class has custom refactor methods.
            Each entry in the dictionary should have as key the name of
            the section in the form that it appears in the docstrings.
            The value should be the postfix of the method, in the
            subclasses, that is responsible for refactoring (e.g.
            {'Methods': 'method'}).

        """
        try:
            self._docstring = lines.splitlines()
        except AttributeError:
            self._docstring = lines
        self.headers = {} if headers is None else headers
        self.bookmarks = []

    def parse(self):
        """ Parse the docstring.

        The docstring is parsed for sections. If a section is found then
        the corresponding refactoring method is called.

        """
        self.index = 0
        self.seek_to_next_non_empty_line()
        while not self.eod:
            header = self.is_section()
            if header:
                self._refactor(header)
            else:
                self.index += 1
                self.seek_to_next_non_empty_line()

    def _refactor(self, header):
        """Call the heading refactor method.

        The header is removed from the docstring and the docstring
        refactoring is dispatched to the appropriate refactoring method.

        The name of the refctoring method is constructed using the form
        _refactor_<header>. Where <header> is the value corresponding to
        ``self.headers[header]``. If there is no custom method for the
        section then the self._refactor_header() is called with the
        found header name as input.

        """
        self.remove_lines(self.index, 2)  # Remove header
        self.remove_if_empty(self.index)  # Remove space after header
        refactor_postfix = self.headers.get(header, 'header')
        method_name = ''.join(('_refactor_', refactor_postfix))
        method = getattr(self, method_name)
        lines = method(header)
        self.insert_and_move(lines, self.index)

    def _refactor_header(self, header):
        """ Refactor the header section using the rubric directive.

        The method has been tested and supports refactoring single word
        headers, two word headers and headers that include a backslash
        ''\''.

        Arguments
        ---------
        header : string
            The header string to use with the rubric directive.

        """
        header = fix_backspace(header)
        directive = '.. rubric:: {0}'.format(header)
        lines = []
        lines += [directive, NEW_LINE]
        return lines

    def extract_items(self, item_class=None):
        """ Extract the definition items from a docstring.

        Parse the items in the description of a section into items of the
        provided class time. Given a DefinitionItem or a subclass defined by
        the ``item_class`` parameter. Staring from the current index position,
        the method checks if in the next two lines a valid  header exists.
        If successful, then the lines that belong to the item description
        block (i.e. header + definition) are poped put from the docstring and
        passed to the ``item_class`` parser and create an instance of
        ``item_class``.

        The process is repeated until there is no compatible ``item_class``
        found or we run out of docstring. Then the method returns a list of
        item_class instances.

        The exit conditions allow for two valid section item layouts:

        1. No lines between items::

            <header1>
                <description1>

                <more description>
            <header2>
                <description2>

        2. One line between items::

            <header1>
                <description1>

                <more description>

            <header2>
                <description2>


        Arguments
        ---------
        item_class : DefinitionItem
            A DefinitionItem or a subclass. This argument is used to check
            if a line in the docstring is a valid item and to parse the
            individual list items in the section. When ``None`` (default) the
            base DefinitionItem class is used.


        Returns
        -------
        parameters : list
            List of the parsed item instances of ``item_class`` type.

        """
        item_type = DefinitionItem if (item_class is None) else item_class
        is_item = item_type.is_definition
        item_blocks = []
        while (not self.eod) and \
                (is_item(self.peek()) or is_item(self.peek(1))):
            self.remove_if_empty(self.index)
            item_blocks.append(self.get_next_block())
        items = [item_type.parse(block) for block in item_blocks]
        return items

    def get_next_block(self):
        """ Get the next item block from the docstring.

        The method reads the next item block in the docstring. The first line
        is assumed to be the DefinitionItem header and the following lines to
        belong to the definition::

            <header line>
                <definition>

        The end of the field is designated by a line with the same indent
        as the field header or two empty lines are found in sequence.

        """
        item_header = self.pop()
        sub_indent = get_indent(item_header) + ' '
        block = [item_header]
        while (not self.eod):
            peek_0 = self.peek()
            peek_1 = self.peek(1)
            if (is_empty(peek_0) and (not peek_1.startswith(sub_indent))) \
                    or ((not is_empty(peek_0)) \
                    and (not peek_0.startswith(sub_indent))):
                break
            else:
                line = self.pop()
                block += [line.rstrip()]
        return block

    def is_section(self):
        """ Check if the current line defines a section.

        .. todo:: split and cleanup this method.

        """
        if self.eod:
            return False

        header = self.peek()
        line2 = self.peek(1)

        # check for underline type format
        underline = underline_regex.match(line2)
        if underline is None:
            return False
        # is the nextline an rst underline?
        striped_header = header.rstrip()
        expected_underline1 = re.sub(r'[A-Za-z\\]|\b\s', '-', striped_header)
        expected_underline2 = re.sub(r'[A-Za-z\\]|\b\s', '=', striped_header)
        if ((underline.group().rstrip() == expected_underline1) or
            (underline.group().rstrip() == expected_underline2)):
            return header.strip()
        else:
            return False

    def insert_lines(self, lines, index):
        """ Insert refactored lines

        Arguments
        ---------
        new_lines : list
            The list of lines to insert

        index : int
            Index to start the insertion
        """
        docstring = self.docstring
        for line in reversed(lines):
            docstring.insert(index, line)

    def insert_and_move(self, lines, index):
        """ Insert refactored lines and move current index to the end.

        """
        self.insert_lines(lines, index)
        self.index += len(lines)

    def seek_to_next_non_empty_line(self):
        """ Goto the next non_empty line.

        """
        docstring = self.docstring
        for line in docstring[self.index:]:
            if not is_empty(line):
                break
            self.index += 1

    def get_next_paragraph(self):
        """ Get the next paragraph designated by an empty line.

        """
        lines = []
        while (not self.eod) and (not is_empty(self.peek())):
            line = self.pop()
            lines.append(line)
        return lines

    def read(self):
        """ Return the next line and advance the index.

        """
        index = self.index
        line = self._docstring[index]
        self.index += 1
        return line

    def remove_lines(self, index, count=1):
        """ Removes the lines from the docstring

        """
        docstring = self.docstring
        del docstring[index:(index + count)]

    def remove_if_empty(self, index=None):
        """ Remove the line from the docstring if it is empty.

        """
        if is_empty(self.docstring[index]):
            self.remove_lines(index)

    def bookmark(self):
        """ append the current index to the end of the list of bookmarks.

        """
        self.bookmarks.append(self.index)

    def goto_bookmark(self, bookmark_index=-1):
        """ Move to bookmark.

        Move the current index to the  docstring line given my the
        ``self.bookmarks[bookmark_index]`` and  remove it fromn the bookmark
        list. Default value will pop the last entry.

        Returns
        -------
        bookmark : int

        """
        self.index = self.bookmarks[bookmark_index]
        return self.bookmarks.pop(bookmark_index)

    def peek(self, ahead=0):
        """ Peek ahead a number of lines

        The function retrieves the line that is ahead of the current
        index. If the index is at the end of the list then it returns an
        empty string.

        Arguments
        ---------
        ahead : int
            The number of lines to look ahead.


        """
        position = self.index + ahead
        try:
            line = self.docstring[position]
        except IndexError:
            line = ''
        return line

    def pop(self, index=None):
        """ Pop a line from the dostrings.

        """
        index = self.index if (index is None) else index
        return self._docstring.pop(index)

    @property
    def eod(self):
        """ End of docstring.

        """
        return self.index >= len(self.docstring)

    @property
    def docstring(self):
        """ Get the docstring lines.

        """
        return self._docstring



# Copyright (c) 2008-2013 by Enthought, Inc.
# All rights reserved.
import os
import re
import subprocess
import sys

from setuptools import setup, Extension, find_packages

MAJOR = 4
MINOR = 6
MICRO = 0

IS_RELEASED = True

VERSION = '%d.%d.%d' % (MAJOR, MINOR, MICRO)


# Return the git revision as a string
def git_version():
    def _minimal_ext_cmd(cmd):
        # construct minimal environment
        env = {}
        for k in ['SYSTEMROOT', 'PATH']:
            v = os.environ.get(k)
            if v is not None:
                env[k] = v
        # LANGUAGE is used on win32
        env['LANGUAGE'] = 'C'
        env['LANG'] = 'C'
        env['LC_ALL'] = 'C'
        out = subprocess.Popen(
            cmd, stdout=subprocess.PIPE, env=env,
        ).communicate()[0]
        return out

    try:
        out = _minimal_ext_cmd(['git', 'describe', '--tags'])
    except OSError:
        out = ''

    git_description = out.strip().decode('ascii')
    expr = r'.*?\-(?P<count>\d+)-g(?P<hash>[a-fA-F0-9]+)'
    match = re.match(expr, git_description)
    if match is None:
        git_revision, git_count = 'Unknown', '0'
    else:
        git_revision, git_count = match.group('hash'), match.group('count')

    return git_revision, git_count


def write_version_py(filename='traits/_version.py'):
    template = """\
# THIS FILE IS GENERATED FROM TRAITS SETUP.PY
version = '{version}'
full_version = '{full_version}'
git_revision = '{git_revision}'
is_released = {is_released}

if not is_released:
    version = full_version
"""
    # Adding the git rev number needs to be done inside
    # write_version_py(), otherwise the import of traits._version messes
    # up the build under Python 3.
    fullversion = VERSION
    if os.path.exists('.git'):
        git_rev, dev_num = git_version()
    elif os.path.exists('traits/_version.py'):
        # must be a source distribution, use existing version file
        try:
            from traits._version import git_revision as git_rev
            from traits._version import full_version as full_v
        except ImportError:
            raise ImportError("Unable to import git_revision. Try removing "
                              "traits/_version.py and the build directory "
                              "before building.")

        match = re.match(r'.*?\.dev(?P<dev_num>\d+)', full_v)
        if match is None:
            dev_num = '0'
        else:
            dev_num = match.group('dev_num')
    else:
        git_rev = 'Unknown'
        dev_num = '0'

    if not IS_RELEASED:
        fullversion += '.dev{0}'.format(dev_num)

    with open(filename, "wt") as fp:
        fp.write(template.format(version=VERSION,
                                 full_version=fullversion,
                                 git_revision=git_rev,
                                 is_released=IS_RELEASED))

if __name__ == "__main__":
    write_version_py()
    from traits import __version__

    ctraits = Extension(
        'traits.ctraits',
        sources=['traits/ctraits.c'],
        extra_compile_args=['-DNDEBUG=1', '-O3'],
        )

    def additional_commands():
        # Pygments 2 isn't supported on Python 3 versions earlier than 3.3, so
        # don't make the documentation command available there.
        if (3,) <= sys.version_info < (3, 3):
            return {}

        try:
            from sphinx.setup_command import BuildDoc
        except ImportError:
            return {}
        else:
            return {'documentation': BuildDoc}

    setup(
        name='traits',
        version=__version__,
        url='http://docs.enthought.com/traits',
        author='David C. Morrill, et. al.',
        author_email='info@enthought.com',
        classifiers=[c.strip() for c in """\
            Development Status :: 5 - Production/Stable
            Intended Audience :: Developers
            Intended Audience :: Science/Research
            License :: OSI Approved :: BSD License
            Operating System :: MacOS
            Operating System :: Microsoft :: Windows
            Operating System :: OS Independent
            Operating System :: POSIX
            Operating System :: Unix
            Programming Language :: C
            Programming Language :: Python
            Topic :: Scientific/Engineering
            Topic :: Software Development
            Topic :: Software Development :: Libraries
            """.splitlines() if len(c.strip()) > 0],
        description='explicitly typed attributes for Python',
        long_description=open('README.rst').read(),
        download_url='https://github.com/enthought/traits',
        ext_modules=[ctraits],
        license='BSD',
        maintainer='ETS Developers',
        maintainer_email='enthought-dev@enthought.com',
        packages=find_packages(exclude=['fixers']),
        platforms=["Windows", "Linux", "Mac OS-X", "Unix", "Solaris"],
        zip_safe=False,
        use_2to3=True,
        use_2to3_fixers=['fixers'],
        # traits_listener.ListenerItem has a trait *next* which gets
        # wrongly renamed
        use_2to3_exclude_fixers=['lib2to3.fixes.fix_next'],
        cmdclass=additional_commands(),
    )



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.

# minimal.py --- Minimal example of using traits.

#--[Imports]-------------------------------------------------------------------
from traits.api import HasTraits, Float, TraitError


#--[Code]----------------------------------------------------------------------
class Person(HasTraits):
    weight = Float(150.0)


#--[Example*]------------------------------------------------------------------
# instantiate the class
joe = Person()

# Show the default value
print joe.weight

# Assign new values
joe.weight = 161.9     # OK to assign a float
print joe.weight

joe.weight = 162       # OK to assign an int
print joe.weight

# The following line causes a traceback:
try:
    joe.weight = 'average'  # Error to assign a string
    print "You should not see this message."
except TraitError:
    print "You can't assign a string to the 'weight' trait."



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.


# all_wildcard.py --- Example of trait attribute wildcard rules

#--[Imports]-------------------------------------------------------------------
from traits.api import Any, Str, Int, HasTraits, TraitError


#--[Code]----------------------------------------------------------------------
class Person(HasTraits):

    # Normal, explicitly defined trait:
    name = Str

    # By default, let all traits have any value:
    _ = Any

    # Except for this one, which must be an Int:
    age = Int


#--[Example*]------------------------------------------------------------------
# Create a sample Person:
bill = Person()

# These assignments should all work:
bill.name = 'William'
bill.address = '121 Drury Lane'
bill.zip_code = 55212
bill.age = 49

# This should generate an error (must be an Int):
print 'Attempting to assign a string to an Int trait object...\n'
try:
    bill.age = 'middle age'
except TraitError, c:
    print 'TraitError: ', c, '\n'

# Display the final results:
bill.print_traits()



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.

# override_default.py -- Example of overriding a default value for
#                        a trait attribute in a subclass

from traits.api import HasTraits, Range, Str


#--[Code]----------------------------------------------------------------------
# Example of overriding a default value for a trait in a subclass:


# Define the base class:
class Employee(HasTraits):

    name = Str
    salary_grade = Range(value=1, low=1, high=10)


# Define a subclass:
class Manager(Employee):

    # Override the default value for the inherited 'salary_grade' trait:
    salary_grade = 5

#--[Example*]------------------------------------------------------------------
# Create an employee and display its initial contents:
joe = Employee(name='Joe')
joe.print_traits()

# Now do the same thing for a manager object:
mike = Manager(name='Mike')
mike.print_traits()
